import json

import click
import structlog

import eiq.platform.entities

from eiq.platform import db

cli_commands = {
    "entity sync-last-updated-at-from-history-events": "sync_last_updated_at"
}

log = structlog.get_logger(__name__)


@click.command(hidden=True)
@click.option(
    "--updated-entities-file",
    type=click.File("wt"),
    required=True,
    help=(
        "The file to write ids to of entities that had out-of-date "
        "last_updated_at timestamp."
    ),
)
@click.option(
    "--dry-run",
    is_flag=True,
    default=False,
    help=("Only write results to the updated-entities-file. "),
)
def sync_last_updated_at(updated_entities_file, dry_run):
    log.info("Executing update query for entity.last_updated_at field")
    rows = db.session.execute(
        """
        WITH entities_from_events AS (
            SELECT
                MAX(timestamp) AS event_timestamp,
                CAST(data -> 'entity' ->> 'id' AS uuid) AS entity_id
            FROM
                history_event
            WHERE
                name = 'entities.meta_updated.by_user'
                AND timestamp > '2019-04-01 00:00:00'
            GROUP BY
                2
        )
        UPDATE entity
        SET last_updated_at = NOW()
        WHERE
                entity.id IN (
                SELECT
                    inner_entity.id
                FROM
                    entities_from_events, entity AS inner_entity
                WHERE
                    inner_entity.id = entity_id
                    AND inner_entity.last_updated_at < event_timestamp
                    AND inner_entity.data ->> 'type' != 'relation'
                 ORDER BY inner_entity.id
                 FOR UPDATE
                 )
        RETURNING entity.id
        """
    )
    entity_ids = [str(row[0]) for row in rows]
    log.info(f"Updated {len(entity_ids)} entities, writing results file.")
    json.dump({"updated_entity_ids": entity_ids}, updated_entities_file)
    if dry_run:
        log.info(f"Dry run, exiting.")
        db.session.rollback()
        return
    db.session.commit()
    pipeline = eiq.platform.entities.IngestionPipeline()
    for entity_id in entity_ids:
        pipeline.notify("entity-changed", entity_id=entity_id)
    log.info(f"Reindexing Elasticsearch and Neo4j.")
    pipeline.run()
